==> cloud-3.2.0.tar.gz: cloud/modules/cloud_service_providers/openstack/tests/src/Traits/OpenStackTestMockTrait.php <==
<?php

namespace Drupal\Tests\openstack\Traits;

use Drupal\Tests\aws_cloud\Traits\AwsCloudTestMockTrait;

/**
 * The trait creating mock data for openstack testing.
 */
trait OpenStackTestMockTrait {

  // Most of functions depends on AwsCloudTestMockTrait.
  use AwsCloudTestMockTrait;

  /**
   * Update describe regions in mock data.
   *
   * @param array $regions
   *   Regions array.
   */
  protected function updateDescribeRegionsMockData(array $regions): void {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeRegions']['Regions'][] = $regions;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update Snapshot mock data.
   *
   * @param int $snapshot_index
   *   The index of Snapshot.
   * @param string $name
   *   The snapshot name.
   * @param bool $is_ec2_service
   *   Whether EC2 service or REST.
   */
  protected function updateSnapshotMockData($snapshot_index, $name, $is_ec2_service = TRUE): void {
    $mock_data = $this->getMockDataFromConfig();

    $is_ec2_service
      ? $mock_data['DescribeSnapshots']['Snapshots'][$snapshot_index]['Tags'][0]
        = ['Key' => 'Name', 'Value' => $name]
      : $mock_data['DescribeSnapshots']['Snapshots'][$snapshot_index]['name']
        = $name;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update describe snapshot in mock data.
   *
   * @param array $test_cases
   *   Test cases array.
   *
   * @throws \Exception
   */
  protected function updateDescribeSnapshotsMockData(array $test_cases): void {
    $random = $this->random;

    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeSnapshots'] = ['Snapshots' => []];
    foreach ($test_cases ?: [] as $test_case) {
      $snapshot = [
        'name' => $test_case['name'] ?? $test_case['id'],
        'SnapshotId' => $test_case['id'],
        'VolumeSize' => 10,
        'Description' => $random->string(32, TRUE),
        'State' => 'completed',
        'VolumeId' => 'vol-' . $this->getRandomId(),
        'Progress' => '100%',
        'Encrypted' => TRUE,
        'KmsKeyId' => 'arn:aws:kms:us-east-1:123456789012:key/6876fb1b-example',
        'OwnerId' => (string) random_int(100000000000, 999999999999),
        'OwnerAlias' => 'amazon',
        'StateMessage' => $random->string(32, TRUE),
        'StartTime' => date('c'),
      ];

      if (!empty($test_case['name'])) {
        $snapshot['Tags'] = [
          ['Key' => 'Name', 'Value' => $test_case['name']],
        ];
      }

      $mock_data['DescribeSnapshots']['Snapshots'][] = $snapshot;
    }

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update mock data related to security group rules.
   *
   * @param array $rules
   *   The security group rules.
   * @param int $rule_type
   *   The security group rule type.
   */
  protected function updateRulesMockData(array $rules, $rule_type): void {
    $mock_data = $this->getMockDataFromConfig();

    $security_group =& $mock_data['DescribeSecurityGroups']['SecurityGroups'][0];
    $security_group['IpPermissions'] = [];
    $security_group['IpPermissionsEgress'] = [];
    foreach ($rules ?: [] as $rule) {
      $permission_name = 'IpPermissions';
      if ($rule['type'] === $rule_type) {
        $permission_name = 'IpPermissionsEgress';
      }

      $permission = [
        'IpProtocol' => 'tcp',
        'FromPort' => $rule['from_port'],
        'ToPort' => $rule['to_port'],
        'RuleId' => $rule['rule_id'] ?? NULL,
      ];

      if ($rule['source'] === 'ip4') {
        $permission['IpRanges'] = [
          [
            'CidrIp' => $rule['cidr_ip'],
            'Description' => $rule['description'],
          ],
        ];
      }
      elseif ($rule['source'] === 'ip6') {
        $permission['Ipv6Ranges'] = [
          [
            'CidrIpv6' => $rule['cidr_ip_v6'],
            'Description' => $rule['description'],
          ],
        ];
      }
      elseif ($rule['source'] === 'group') {
        $permission['UserIdGroupPairs'] = [
          [
            'UserId' => $rule['user_id'],
            'GroupId' => $rule['group_id'],
            'VpcId' => $rule['vpc_id'],
            'VpcPeeringConnectionId' => $rule['peering_connection_id'],
            'PeeringStatus' => $rule['peering_status'],
            'Description' => $rule['description'],
          ],
        ];
      }

      $security_group[$permission_name][] = $permission;
    }

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Add describe flavors in mock data.
   *
   * @param int $id
   *   Flavor ID.
   * @param string $name
   *   Flavor Name.
   */
  protected function addDescribeFlavorsMockData($id, $name): void {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeFlavors'] = [
      [
        'id' => $id,
        'name' => $name,
      ],
    ];
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Add auth token in mock data.
   */
  protected function addAuthTokenMockData($status_code = ''): void {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['GenerateAuthToken'] = [
      'status_code' => $status_code ?? 200,
      'auth_token' => $this->random->name(32, TRUE),
    ];
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update auth token in mock data.
   *
   * @param string $status_code
   *   The status code of auth token.
   */
  protected function updateAuthTokenMockData($status_code = ''): void {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['GenerateAuthToken']['status_code'] = $status_code ?? 200;
    $this->updateMockDataToConfig($mock_data);
  }

}

==> cloud-4.0.0.tar.gz: cloud/modules/cloud_service_providers/openstack/tests/src/Traits/OpenStackTestMockTrait.php <==
<?php

namespace Drupal\Tests\openstack\Traits;

use Drupal\Tests\aws_cloud\Traits\AwsCloudTestMockTrait;

/**
 * The trait creating mock data for openstack testing.
 */
trait OpenStackTestMockTrait {

  // Most of functions depends on AwsCloudTestMockTrait.
  use AwsCloudTestMockTrait;

  /**
   * Update describe regions in mock data.
   *
   * @param array $regions
   *   Regions array.
   */
  protected function updateDescribeRegionsMockData(array $regions): void {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeRegions']['Regions'][] = $regions;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update describe security groups in mock data.
   *
   * @param array $security_groups
   *   Security groups array.
   */
  protected function updateDescribeSecurityGroupsMockData(array $security_groups): void {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeSecurityGroups']['SecurityGroups'] = $security_groups;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update the CreateSecurityGroup mock data.
   */
  protected function updateCreateSecurityGroupMockData() {
    $mock_data = $this->getMockDataFromConfig();
    $vars = $this->getMockDataTemplateVars();
    $mock_data['CreateSecurityGroup']['GroupId'] = $vars['group_id'];
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update Snapshot mock data.
   *
   * @param int $snapshot_index
   *   The index of Snapshot.
   * @param string $name
   *   The snapshot name.
   * @param bool $is_ec2_service
   *   Whether EC2 service or REST.
   */
  protected function updateSnapshotMockData($snapshot_index, $name, $is_ec2_service = TRUE): void {
    $mock_data = $this->getMockDataFromConfig();

    $is_ec2_service
      ? $mock_data['DescribeSnapshots']['Snapshots'][$snapshot_index]['Tags'][0]
        = ['Key' => 'Name', 'Value' => $name]
      : $mock_data['DescribeSnapshots']['Snapshots'][$snapshot_index]['name']
        = $name;

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update describe snapshot in mock data.
   *
   * @param array $test_cases
   *   Test cases array.
   * @param bool $is_ec2_service
   *   Whether EC2 service or REST.
   * @param string $cloud_context
   *   The cloud context.
   * @param int $uid
   *   The uid.
   *
   * @throws \Exception
   */
  protected function updateDescribeSnapshotsMockData(array $test_cases, $is_ec2_service = FALSE, $cloud_context = '', $uid = 0): void {
    $random = $this->random;

    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeSnapshots'] = ['Snapshots' => []];
    foreach ($test_cases ?: [] as $test_case) {
      $snapshot = [
        'name' => $test_case['name'] ?? $test_case['id'],
        'SnapshotId' => $test_case['id'],
        'VolumeSize' => 10,
        'Description' => $random->string(32, TRUE),
        'State' => 'completed',
        'VolumeId' => 'vol-' . $this->getRandomId(),
        'Progress' => '100%',
        'Encrypted' => TRUE,
        'KmsKeyId' => 'arn:aws:kms:us-east-1:123456789012:key/6876fb1b-example',
        'OwnerId' => (string) random_int(100000000000, 999999999999),
        'OwnerAlias' => 'amazon',
        'StateMessage' => $random->string(32, TRUE),
        'StartTime' => date('c'),
      ];

      if (!empty($is_ec2_service) && !empty($test_case['name'])) {
        $snapshot['Tags'] = [
          ['Key' => 'Name', 'Value' => $test_case['name']],
        ];
      }
      if (empty($is_ec2_service)) {
        $snapshot['Tags'] = [
          \Drupal::service('cloud')->getTagCreatedByUidValue('openstack', $cloud_context, $uid),
        ];
      }
      $mock_data['DescribeSnapshots']['Snapshots'][] = $snapshot;
    }

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update mock data related to security group rules.
   *
   * @param array $rules
   *   The security group rules.
   * @param int $rule_type
   *   The security group rule type.
   */
  protected function updateRulesMockData(array $rules, $rule_type): void {
    $mock_data = $this->getMockDataFromConfig();

    $security_group =& $mock_data['DescribeSecurityGroups']['SecurityGroups'][0];
    $security_group['IpPermissions'] = [];
    $security_group['IpPermissionsEgress'] = [];
    foreach ($rules ?: [] as $rule) {
      $permission_name = 'IpPermissions';
      if ($rule['type'] === $rule_type) {
        $permission_name = 'IpPermissionsEgress';
      }

      $permission = [
        'IpProtocol' => 'tcp',
        'FromPort' => $rule['from_port'],
        'ToPort' => $rule['to_port'],
        'RuleId' => $rule['rule_id'] ?? NULL,
      ];

      if ($rule['source'] === 'ip4') {
        $permission['IpRanges'] = [
          [
            'CidrIp' => $rule['cidr_ip'],
            'Description' => $rule['description'],
          ],
        ];
      }
      elseif ($rule['source'] === 'ip6') {
        $permission['Ipv6Ranges'] = [
          [
            'CidrIpv6' => $rule['cidr_ip_v6'],
            'Description' => $rule['description'],
          ],
        ];
      }
      elseif ($rule['source'] === 'group') {
        $permission['UserIdGroupPairs'] = [
          [
            'UserId' => $rule['user_id'],
            'GroupId' => $rule['group_id'],
            'VpcId' => $rule['vpc_id'],
            'VpcPeeringConnectionId' => $rule['peering_connection_id'],
            'PeeringStatus' => $rule['peering_status'],
            'Description' => $rule['description'],
          ],
        ];
      }

      $security_group[$permission_name][] = $permission;
    }

    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Add describe flavors in mock data.
   *
   * @param int $id
   *   Flavor ID.
   * @param string $name
   *   Flavor Name.
   */
  protected function addDescribeFlavorsMockData($id, $name): void {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeFlavors'] = [
      [
        'id' => $id,
        'name' => $name,
      ],
    ];
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Add auth token in mock data.
   */
  protected function addAuthTokenMockData($status_code = ''): void {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['GenerateAuthToken'] = [
      'status_code' => $status_code ?? 200,
      'auth_token' => $this->random->name(32, TRUE),
    ];
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Update auth token in mock data.
   *
   * @param string $status_code
   *   The status code of auth token.
   */
  protected function updateAuthTokenMockData($status_code = ''): void {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['GenerateAuthToken']['status_code'] = $status_code ?? 200;
    $this->updateMockDataToConfig($mock_data);
  }

  /**
   * Add Volume mock data.
   *
   * @param array $data
   *   Array of volume data.
   * @param string $tag_created_uid
   *   The tag created by uid in mock data.
   * @param bool $is_ec2_service
   *   Whether EC2 service or REST.
   * @param string $cloud_context
   *   The cloud context.
   * @param int $uid
   *   The uid.
   *
   * @throws \Exception
   */
  protected function addVolumeMockData(array $data, $tag_created_uid, $is_ec2_service = FALSE, $cloud_context = '', $uid = 0): void {
    $mock_data = $this->getMockDataFromConfig();
    $vars = $this->getMockDataTemplateVars();

    $volume = [
      'VolumeId' => $data['VolumeId'] ?? $data['name'],
      'Attachments' => [
        ['InstanceId' => NULL],
      ],
      'State' => 'available',
      'SnapshotId' => $data['snapshot_id'],
      'Size' => $data['size'],
      'VirtualizationType' => NULL,
      'VolumeType' => $data['volume_type'],
      'Iops' => $data['iops'] ?? '',
      'AvailabilityZone' => $data['availability_zone'],
      'Encrypted' => $data['encrypted'] ?? '',
      'KmsKeyId' => NULL,
      'CreateTime' => $vars['create_time'],
    ];

    $volume['Tags'] = !empty($is_ec2_service)
      ? [
          [
            'Key' => $tag_created_uid,
            'Value' => $uid,
          ],
      ]
      : [
        \Drupal::service('cloud')->getTagCreatedByUidValue('openstack', $cloud_context, $uid),
      ];

    $mock_data['DescribeVolumes']['Volumes'][] = $volume;
    $this->updateMockDataToConfig($mock_data);

    $snapshot_name = 'snapshot-name' . $this->random->name(10, TRUE);
    $this->updateDescribeSnapshotsMockData(
      [
        [
          'id' => $data['snapshot_id'],
          'name' => $snapshot_name,
        ],
      ],
      $is_ec2_service,
      $cloud_context,
      $uid,
    );
  }

}
